@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef __GST_CONTROL_BINDING_H__
@@ -47,19 +47,13 @@ G_BEGIN_DECLS
 typedef struct _GstControlBinding GstControlBinding;
 typedef struct _GstControlBindingClass GstControlBindingClass;
 
-/**
- * GstControlBindingConvert:
- * @self: the #GstControlBinding instance
- * @src_value: the value returned by the cotnrol source
- * @dest_value: the target GValue
- *
- * Function to map a control-value to the target GValue.
- */
-typedef void (* GstControlBindingConvert) (GstControlBinding *self, gdouble src_value, GValue *dest_value);
+/* FIXME(2.0): remove, this is unused */
+typedef void (* GstControlBindingConvert) (GstControlBinding *binding, gdouble src_value, GValue *dest_value);
 
 /**
  * GstControlBinding:
  * @name: name of the property of this binding
+ * @pspec: #GParamSpec for this property
  *
  * The instance structure of #GstControlBinding.
  */
@@ -67,8 +61,8 @@ struct _GstControlBinding {
   GstObject parent;
   
   /*< public >*/
-  gchar *name;                  /* name of the property */
-  GParamSpec *pspec;            /* GParamSpec for this property */
+  gchar *name;
+  GParamSpec *pspec;
 
   /*< private >*/
   GstObject *object;            /* GstObject owning the property
@@ -81,7 +75,6 @@ struct _GstControlBinding {
 /**
  * GstControlBindingClass:
  * @parent_class: Parent class
- * @convert: Class method to convert control-values
  *
  * The class structure of #GstControlBinding.
  */
@@ -91,10 +84,10 @@ struct _GstControlBindingClass
   GstObjectClass parent_class;
 
   /* virtual methods */
-  gboolean (* sync_values) (GstControlBinding *self, GstObject *object, GstClockTime timestamp, GstClockTime last_sync);
-  GValue * (* get_value) (GstControlBinding *self, GstClockTime timestamp);
-  gboolean (* get_value_array) (GstControlBinding *self, GstClockTime timestamp,GstClockTime interval, guint n_values, gpointer values);
-  gboolean (* get_g_value_array) (GstControlBinding *self, GstClockTime timestamp,GstClockTime interval, guint n_values, GValue *values);
+  gboolean (* sync_values) (GstControlBinding *binding, GstObject *object, GstClockTime timestamp, GstClockTime last_sync);
+  GValue * (* get_value) (GstControlBinding *binding, GstClockTime timestamp);
+  gboolean (* get_value_array) (GstControlBinding *binding, GstClockTime timestamp,GstClockTime interval, guint n_values, gpointer values);
+  gboolean (* get_g_value_array) (GstControlBinding *binding, GstClockTime timestamp,GstClockTime interval, guint n_values, GValue *values);
 
   /*< private >*/
   gpointer _gst_reserved[GST_PADDING];
@@ -106,7 +99,7 @@ GType gst_control_binding_get_type (void);
 
 /* Functions */
 
-gboolean            gst_control_binding_sync_values        (GstControlBinding * self, GstObject *object, 
+gboolean            gst_control_binding_sync_values        (GstControlBinding * binding, GstObject *object,
                                                             GstClockTime timestamp, GstClockTime last_sync);
 GValue *            gst_control_binding_get_value          (GstControlBinding *binding,
                                                             GstClockTime timestamp);
@@ -115,8 +108,8 @@ gboolean            gst_control_binding_get_value_array    (GstControlBinding *b
 gboolean            gst_control_binding_get_g_value_array  (GstControlBinding *binding, GstClockTime timestamp,
                                                             GstClockTime interval, guint n_values, GValue *values);
 
-void                gst_control_binding_set_disabled       (GstControlBinding * self, gboolean disabled);
-gboolean            gst_control_binding_is_disabled        (GstControlBinding * self);
+void                gst_control_binding_set_disabled       (GstControlBinding * binding, gboolean disabled);
+gboolean            gst_control_binding_is_disabled        (GstControlBinding * binding);
 G_END_DECLS
 
 #endif /* __GST_CONTROL_BINDING_H__ */
