@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 /**
  * SECTION:gstcollectpads
@@ -55,7 +55,7 @@
  *   </para></listitem>
  *   <listitem><para>
  *     Data can also be dequeued in byte units using the gst_collect_pads_available(),
- *     gst_collect_pads_read() and gst_collect_pads_flush() calls.
+ *     gst_collect_pads_read_buffer() and gst_collect_pads_flush() calls.
  *   </para></listitem>
  *   <listitem><para>
  *     Elements should call gst_collect_pads_start() and gst_collect_pads_stop() in
@@ -78,8 +78,6 @@
  * </itemizedlist>
  *
  * Last reviewed on 2011-10-28 (0.10.36)
- *
- * Since: 0.10.36
  */
 
 #ifdef HAVE_CONFIG_H
@@ -288,13 +286,11 @@ gst_collect_pads_finalize (GObject * object)
 /**
  * gst_collect_pads_new:
  *
- * Create a new instance of #GstCollectsPads.
+ * Create a new instance of #GstCollectPads.
  *
  * MT safe.
  *
  * Returns: (transfer full): a new #GstCollectPads, or NULL in case of an error.
- *
- * Since: 0.10.36
  */
 GstCollectPads *
 gst_collect_pads_new (void)
@@ -322,11 +318,11 @@ gst_collect_pads_set_buffer_function_locked (GstCollectPads * pads,
  * @user_data: (closure): user data passed to the function
  *
  * Set the callback function and user data that will be called with
- * the oldest buffer when all pads have been collected.
+ * the oldest buffer when all pads have been collected, or NULL on EOS.
+ * If a buffer is passed, the callback owns a reference and must unref
+ * it.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
 gst_collect_pads_set_buffer_function (GstCollectPads * pads,
@@ -349,8 +345,6 @@ gst_collect_pads_set_buffer_function (GstCollectPads * pads,
  * Set the timestamp comparison function.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 /* NOTE allowing to change comparison seems not advisable;
 no known use-case, and collaboration with default algorithm is unpredictable.
@@ -371,7 +365,7 @@ gst_collect_pads_set_compare_function (GstCollectPads * pads,
 
 /**
  * gst_collect_pads_set_function:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @func: the function to set
  * @user_data: user data passed to the function
  *
@@ -386,8 +380,6 @@ gst_collect_pads_set_compare_function (GstCollectPads * pads,
  * If this callback is set, the former will be unset.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
 gst_collect_pads_set_function (GstCollectPads * pads,
@@ -433,7 +425,7 @@ unref_data (GstCollectData * data)
 
 /**
  * gst_collect_pads_set_event_function:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @func: the function to set
  * @user_data: user data passed to the function
  *
@@ -445,8 +437,6 @@ unref_data (GstCollectData * data)
  * if so (unusually) needed.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
 gst_collect_pads_set_event_function (GstCollectPads * pads,
@@ -463,7 +453,7 @@ gst_collect_pads_set_event_function (GstCollectPads * pads,
 
 /**
  * gst_collect_pads_set_query_function:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @func: the function to set
  * @user_data: user data passed to the function
  *
@@ -475,8 +465,6 @@ gst_collect_pads_set_event_function (GstCollectPads * pads,
  * if so (unusually) needed.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
 gst_collect_pads_set_query_function (GstCollectPads * pads,
@@ -493,16 +481,14 @@ gst_collect_pads_set_query_function (GstCollectPads * pads,
 
 /**
 * gst_collect_pads_clip_running_time:
-* @pads: the collectspads to use
+* @pads: the collectpads to use
 * @cdata: collect data of corresponding pad
 * @buf: buffer being clipped
-* @outbuf: output buffer with running time, or NULL if clipped
+* @outbuf: (allow-none): output buffer with running time, or NULL if clipped
 * @user_data: user data (unused)
 *
 * Convenience clipping function that converts incoming buffer's timestamp
 * to running time, or clips the buffer if outside configured segment.
-*
-* Since: 0.10.37
 */
 GstFlowReturn
 gst_collect_pads_clip_running_time (GstCollectPads * pads,
@@ -512,21 +498,24 @@ gst_collect_pads_clip_running_time (GstCollectPads * pads,
   GstClockTime time;
 
   *outbuf = buf;
-  time = GST_BUFFER_TIMESTAMP (buf);
+  time = GST_BUFFER_PTS (buf);
 
   /* invalid left alone and passed */
   if (G_LIKELY (GST_CLOCK_TIME_IS_VALID (time))) {
     time = gst_segment_to_running_time (&cdata->segment, GST_FORMAT_TIME, time);
     if (G_UNLIKELY (!GST_CLOCK_TIME_IS_VALID (time))) {
-      GST_DEBUG_OBJECT (cdata->pad, "clipping buffer on pad outside segment");
+      GST_DEBUG_OBJECT (cdata->pad, "clipping buffer on pad outside segment %"
+          GST_TIME_FORMAT, GST_TIME_ARGS (GST_BUFFER_PTS (buf)));
       gst_buffer_unref (buf);
       *outbuf = NULL;
     } else {
       GST_LOG_OBJECT (cdata->pad, "buffer ts %" GST_TIME_FORMAT " -> %"
           GST_TIME_FORMAT " running time",
-          GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)), GST_TIME_ARGS (time));
+          GST_TIME_ARGS (GST_BUFFER_PTS (buf)), GST_TIME_ARGS (time));
       *outbuf = gst_buffer_make_writable (buf);
-      GST_BUFFER_TIMESTAMP (*outbuf) = time;
+      GST_BUFFER_PTS (*outbuf) = time;
+      GST_BUFFER_DTS (*outbuf) = gst_segment_to_running_time (&cdata->segment,
+          GST_FORMAT_TIME, GST_BUFFER_DTS (*outbuf));
     }
   }
 
@@ -535,14 +524,12 @@ gst_collect_pads_clip_running_time (GstCollectPads * pads,
 
  /**
  * gst_collect_pads_set_clip_function:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @clipfunc: clip function to install
  * @user_data: user data to pass to @clip_func
  *
  * Install a clipping function that is called right after a buffer is received
- * on a pad managed by @pads. See #GstCollectPad2ClipFunction for more info.
- *
- * Since: 0.10.36
+ * on a pad managed by @pads. See #GstCollectPadsClipFunction for more info.
  */
 void
 gst_collect_pads_set_clip_function (GstCollectPads * pads,
@@ -557,40 +544,7 @@ gst_collect_pads_set_clip_function (GstCollectPads * pads,
 
 /**
  * gst_collect_pads_add_pad:
- * @pads: the collectspads to use
- * @pad: (transfer none): the pad to add
- * @size: the size of the returned #GstCollectData structure
- *
- * Add a pad to the collection of collect pads. The pad has to be
- * a sinkpad. The refcount of the pad is incremented. Use
- * gst_collect_pads_remove_pad() to remove the pad from the collection
- * again.
- *
- * You specify a size for the returned #GstCollectData structure
- * so that you can use it to store additional information.
- *
- * The pad will be automatically activated in push mode when @pads is
- * started.
- *
- * This function calls gst_collect_pads_add_pad_full() passing a value of NULL
- * for destroy_notify and TRUE for locked.
- *
- * MT safe.
- *
- * Returns: a new #GstCollectData to identify the new pad. Or NULL
- *   if wrong parameters are supplied.
- *
- * Since: 0.10.36
- */
-GstCollectData *
-gst_collect_pads_add_pad (GstCollectPads * pads, GstPad * pad, guint size)
-{
-  return gst_collect_pads_add_pad_full (pads, pad, size, NULL, TRUE);
-}
-
-/**
- * gst_collect_pads_add_pad_full:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @pad: (transfer none): the pad to add
  * @size: the size of the returned #GstCollectData structure
  * @destroy_notify: function to be called before the returned #GstCollectData
@@ -623,14 +577,12 @@ gst_collect_pads_add_pad (GstCollectPads * pads, GstPad * pad, guint size)
  *
  * MT safe.
  *
- * Since: 0.10.36
- *
  * Returns: a new #GstCollectData to identify the new pad. Or NULL
  *   if wrong parameters are supplied.
  */
 GstCollectData *
-gst_collect_pads_add_pad_full (GstCollectPads * pads, GstPad * pad,
-    guint size, GstCollectDataDestroyNotify destroy_notify, gboolean lock)
+gst_collect_pads_add_pad (GstCollectPads * pads, GstPad * pad, guint size,
+    GstCollectDataDestroyNotify destroy_notify, gboolean lock)
 {
   GstCollectData *data;
 
@@ -689,7 +641,7 @@ find_pad (GstCollectData * data, GstPad * pad)
 
 /**
  * gst_collect_pads_remove_pad:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @pad: (transfer none): the pad to remove
  *
  * Remove a pad from the collection of collect pads. This function will also
@@ -701,8 +653,6 @@ find_pad (GstCollectData * data, GstPad * pad)
  * MT safe.
  *
  * Returns: %TRUE if the pad could be removed.
- *
- * Since: 0.10.36
  */
 gboolean
 gst_collect_pads_remove_pad (GstCollectPads * pads, GstPad * pad)
@@ -776,91 +726,8 @@ unknown_pad:
   }
 }
 
-/**
- * gst_collect_pads_is_active:
- * @pads: the collectspads to use
- * @pad: the pad to check
- *
- * Check if a pad is active.
- *
- * This function is currently not implemented.
- *
- * MT safe.
- *
- * Returns: %TRUE if the pad is active.
- *
- * Since: 0.10.36
- */
-gboolean
-gst_collect_pads_is_active (GstCollectPads * pads, GstPad * pad)
-{
-  g_return_val_if_fail (pads != NULL, FALSE);
-  g_return_val_if_fail (GST_IS_COLLECT_PADS (pads), FALSE);
-  g_return_val_if_fail (pad != NULL, FALSE);
-  g_return_val_if_fail (GST_IS_PAD (pad), FALSE);
-
-  g_warning ("gst_collect_pads_is_active() is not implemented");
-
-  return FALSE;
-}
-
-/**
- * gst_collect_pads_collect:
- * @pads: the collectspads to use
- *
- * Collect data on all pads. This function is usually called
- * from a #GstTask function in an element. 
- *
- * This function is currently not implemented.
- *
- * MT safe.
- *
- * Returns: #GstFlowReturn of the operation.
- *
- * Since: 0.10.36
- */
-GstFlowReturn
-gst_collect_pads_collect (GstCollectPads * pads)
-{
-  g_return_val_if_fail (pads != NULL, GST_FLOW_ERROR);
-  g_return_val_if_fail (GST_IS_COLLECT_PADS (pads), GST_FLOW_ERROR);
-
-  g_warning ("gst_collect_pads_collect() is not implemented");
-
-  return GST_FLOW_NOT_SUPPORTED;
-}
-
-/**
- * gst_collect_pads_collect_range:
- * @pads: the collectspads to use
- * @offset: the offset to collect
- * @length: the length to collect
- *
- * Collect data with @offset and @length on all pads. This function
- * is typically called in the getrange function of an element. 
- *
- * This function is currently not implemented.
- *
- * MT safe.
- *
- * Returns: #GstFlowReturn of the operation.
- *
- * Since: 0.10.36
- */
-GstFlowReturn
-gst_collect_pads_collect_range (GstCollectPads * pads, guint64 offset,
-    guint length)
-{
-  g_return_val_if_fail (pads != NULL, GST_FLOW_ERROR);
-  g_return_val_if_fail (GST_IS_COLLECT_PADS (pads), GST_FLOW_ERROR);
-
-  g_warning ("gst_collect_pads_collect_range() is not implemented");
-
-  return GST_FLOW_NOT_SUPPORTED;
-}
-
 /*
- * Must be called with STREAM_LOCK.
+ * Must be called with STREAM_LOCK and OBJECT_LOCK.
  */
 static void
 gst_collect_pads_set_flushing_unlocked (GstCollectPads * pads,
@@ -869,7 +736,7 @@ gst_collect_pads_set_flushing_unlocked (GstCollectPads * pads,
   GSList *walk = NULL;
 
   /* Update the pads flushing flag */
-  for (walk = pads->data; walk; walk = g_slist_next (walk)) {
+  for (walk = pads->priv->pad_list; walk; walk = g_slist_next (walk)) {
     GstCollectData *cdata = walk->data;
 
     if (GST_IS_PAD (cdata->pad)) {
@@ -893,7 +760,7 @@ gst_collect_pads_set_flushing_unlocked (GstCollectPads * pads,
 
 /**
  * gst_collect_pads_set_flushing:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @flushing: desired state of the pads
  *
  * Change the flushing state of all the pads in the collection. No pad
@@ -903,8 +770,6 @@ gst_collect_pads_set_flushing_unlocked (GstCollectPads * pads,
  * e.g. by sending a FLUSH_START downstream.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
 gst_collect_pads_set_flushing (GstCollectPads * pads, gboolean flushing)
@@ -914,19 +779,19 @@ gst_collect_pads_set_flushing (GstCollectPads * pads, gboolean flushing)
 
   /* NOTE since this eventually calls _pop, some (STREAM_)LOCK is needed here */
   GST_COLLECT_PADS_STREAM_LOCK (pads);
+  GST_OBJECT_LOCK (pads);
   gst_collect_pads_set_flushing_unlocked (pads, flushing);
+  GST_OBJECT_UNLOCK (pads);
   GST_COLLECT_PADS_STREAM_UNLOCK (pads);
 }
 
 /**
  * gst_collect_pads_start:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  *
  * Starts the processing of data in the collect_pads.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
 gst_collect_pads_start (GstCollectPads * pads)
@@ -963,14 +828,12 @@ gst_collect_pads_start (GstCollectPads * pads)
 
 /**
  * gst_collect_pads_stop:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  *
  * Stops the processing of data in the collect_pads. this function
  * will also unblock any blocking operations.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
 gst_collect_pads_stop (GstCollectPads * pads)
@@ -1023,7 +886,7 @@ gst_collect_pads_stop (GstCollectPads * pads)
 
 /**
  * gst_collect_pads_peek:
- * @pads: the collectspads to peek
+ * @pads: the collectpads to peek
  * @data: the data to use
  *
  * Peek at the buffer currently queued in @data. This function
@@ -1034,8 +897,6 @@ gst_collect_pads_stop (GstCollectPads * pads)
  *
  * Returns: The buffer in @data or NULL if no buffer is queued.
  *  should unref the buffer after usage.
- *
- * Since: 0.10.36
  */
 GstBuffer *
 gst_collect_pads_peek (GstCollectPads * pads, GstCollectData * data)
@@ -1057,7 +918,7 @@ gst_collect_pads_peek (GstCollectPads * pads, GstCollectData * data)
 
 /**
  * gst_collect_pads_pop:
- * @pads: the collectspads to pop
+ * @pads: the collectpads to pop
  * @data: the data to use
  *
  * Pop the buffer currently queued in @data. This function
@@ -1068,8 +929,6 @@ gst_collect_pads_peek (GstCollectPads * pads, GstCollectData * data)
  *
  * Returns: (transfer full): The buffer in @data or NULL if no buffer was
  *   queued. You should unref the buffer after usage.
- *
- * Since: 0.10.36
  */
 GstBuffer *
 gst_collect_pads_pop (GstCollectPads * pads, GstCollectData * data)
@@ -1109,7 +968,7 @@ gst_collect_pads_clear (GstCollectPads * pads, GstCollectData * data)
 
 /**
  * gst_collect_pads_available:
- * @pads: the collectspads to query
+ * @pads: the collectpads to query
  *
  * Query how much bytes can be read from each queued buffer. This means
  * that the result of this call is the maximum number of bytes that can
@@ -1122,8 +981,6 @@ gst_collect_pads_clear (GstCollectPads * pads, GstCollectData * data)
  *
  * Returns: The maximum number of bytes queued on all pads. This function
  * returns 0 if a pad has no queued buffer.
- *
- * Since: 0.10.36
  */
 /* we might pre-calculate this in some struct field,
  * but would then have to maintain this in _chain and particularly _pop, etc,
@@ -1180,7 +1037,7 @@ not_filled:
 
 /**
  * gst_collect_pads_flush:
- * @pads: the collectspads to query
+ * @pads: the collectpads to query
  * @data: the data to use
  * @size: the number of bytes to flush
  *
@@ -1193,8 +1050,6 @@ not_filled:
  *
  * Returns: The number of bytes flushed This can be less than @size and
  * is 0 if the pad was end-of-stream.
- *
- * Since: 0.10.36
  */
 guint
 gst_collect_pads_flush (GstCollectPads * pads, GstCollectData * data,
@@ -1228,7 +1083,7 @@ gst_collect_pads_flush (GstCollectPads * pads, GstCollectData * data,
 
 /**
  * gst_collect_pads_read_buffer:
- * @pads: the collectspads to query
+ * @pads: the collectpads to query
  * @data: the data to use
  * @size: the number of bytes to read
  *
@@ -1239,8 +1094,6 @@ gst_collect_pads_flush (GstCollectPads * pads, GstCollectData * data,
  *
  * MT safe.
  *
- * Since: 0.10.36
- *
  * Returns: (transfer full): A sub buffer. The size of the buffer can be less that requested.
  * A return of NULL signals that the pad is end-of-stream.
  * Unref the buffer after use.
@@ -1268,7 +1121,7 @@ gst_collect_pads_read_buffer (GstCollectPads * pads, GstCollectData * data,
 
 /**
  * gst_collect_pads_take_buffer:
- * @pads: the collectspads to query
+ * @pads: the collectpads to query
  * @data: the data to use
  * @size: the number of bytes to read
  *
@@ -1280,8 +1133,6 @@ gst_collect_pads_read_buffer (GstCollectPads * pads, GstCollectData * data,
  *
  * MT safe.
  *
- * Since: 0.10.36
- *
  * Returns: A sub buffer. The size of the buffer can be less that requested.
  * A return of NULL signals that the pad is end-of-stream.
  * Unref the buffer after use.
@@ -1300,7 +1151,7 @@ gst_collect_pads_take_buffer (GstCollectPads * pads, GstCollectData * data,
 
 /**
  * gst_collect_pads_set_waiting:
- * @pads: the collectspads
+ * @pads: the collectpads
  * @data: the data to use
  * @waiting: boolean indicating whether this pad should operate
  *           in waiting or non-waiting mode
@@ -1313,8 +1164,6 @@ gst_collect_pads_take_buffer (GstCollectPads * pads, GstCollectData * data,
  * in the callback.
  *
  * MT safe.
- *
- * Since: 0.10.36
  */
 void
 gst_collect_pads_set_waiting (GstCollectPads * pads, GstCollectData * data,
@@ -1436,7 +1285,9 @@ gst_collect_pads_check_collected (GstCollectPads * pads)
     GST_DEBUG_OBJECT (pads, "All active pads (%d) are EOS, calling %s",
         pads->priv->numpads, GST_DEBUG_FUNCPTR_NAME (func));
 
-    flow_ret = func (pads, user_data);
+    do {
+      flow_ret = func (pads, user_data);
+    } while (flow_ret == GST_FLOW_OK);
   } else {
     gboolean collected = FALSE;
 
@@ -1494,6 +1345,7 @@ gst_collect_pads_recalculate_waiting (GstCollectPads * pads)
   for (collected = pads->data; collected; collected = g_slist_next (collected)) {
     GstCollectData *data = (GstCollectData *) collected->data;
     int cmp_res;
+    GstClockTime comp_time;
 
     /* check if pad has a segment */
     if (data->segment.format == GST_FORMAT_UNDEFINED) {
@@ -1510,7 +1362,8 @@ gst_collect_pads_recalculate_waiting (GstCollectPads * pads)
     }
 
     /* check if the waiting state should be changed */
-    cmp_res = pads->priv->compare_func (pads, data, data->segment.start,
+    comp_time = data->segment.position;
+    cmp_res = pads->priv->compare_func (pads, data, comp_time,
         pads->priv->earliest_data, pads->priv->earliest_time,
         pads->priv->compare_user_data);
     if (cmp_res > 0)
@@ -1539,8 +1392,6 @@ gst_collect_pads_recalculate_waiting (GstCollectPads * pads)
  *
  * This function should be called with STREAM_LOCK held,
  * such as in the callback.
- *
- * Since: 0.10.36
  */
 static void
 gst_collect_pads_find_best_pad (GstCollectPads * pads,
@@ -1561,7 +1412,10 @@ gst_collect_pads_find_best_pad (GstCollectPads * pads,
     buffer = gst_collect_pads_peek (pads, data);
     /* if we have a buffer check if it is better then the current best one */
     if (buffer != NULL) {
-      timestamp = GST_BUFFER_TIMESTAMP (buffer);
+      timestamp = GST_BUFFER_DTS (buffer);
+      if (!GST_CLOCK_TIME_IS_VALID (timestamp)) {
+        timestamp = GST_BUFFER_PTS (buffer);
+      }
       gst_buffer_unref (buffer);
       if (best == NULL || pads->priv->compare_func (pads, data, timestamp,
               best, best_time, pads->priv->compare_user_data) < 0) {
@@ -1687,9 +1541,63 @@ gst_collect_pads_default_compare_func (GstCollectPads * pads,
   return 0;
 }
 
+/* called with STREAM_LOCK */
+static void
+gst_collect_pads_handle_position_update (GstCollectPads * pads,
+    GstCollectData * data, GstClockTime new_pos)
+{
+  gint cmp_res;
+
+  /* If oldest time is not known, or current pad got newsegment;
+   * recalculate the state */
+  if (!pads->priv->earliest_data || pads->priv->earliest_data == data) {
+    gst_collect_pads_recalculate_full (pads);
+    goto exit;
+  }
+
+  /* Check if the waiting state of the pad should change. */
+  cmp_res =
+      pads->priv->compare_func (pads, data, new_pos,
+      pads->priv->earliest_data, pads->priv->earliest_time,
+      pads->priv->compare_user_data);
+
+  if (cmp_res > 0)
+    /* Stop waiting */
+    gst_collect_pads_set_waiting (pads, data, FALSE);
+
+exit:
+  return;
+
+}
+
+static GstClockTime
+gst_collect_pads_clip_time (GstCollectPads * pads, GstCollectData * data,
+    GstClockTime time)
+{
+  GstClockTime otime = time;
+  GstBuffer *in, *out = NULL;
+
+  if (pads->priv->clip_func) {
+    in = gst_buffer_new ();
+    GST_BUFFER_PTS (in) = time;
+    GST_BUFFER_DTS (in) = time;
+    pads->priv->clip_func (pads, data, in, &out, pads->priv->clip_user_data);
+    if (out) {
+      otime = GST_BUFFER_PTS (out);
+      gst_buffer_unref (out);
+    } else {
+      /* FIXME should distinguish between ahead or after segment,
+       * let's assume after segment and use some large time ... */
+      otime = G_MAXINT64 / 2;
+    }
+  }
+
+  return otime;
+}
+
 /**
  * gst_collect_pads_event_default:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @data: collect data of corresponding pad
  * @event: event being processed
  * @discard: process but do not send event downstream
@@ -1697,8 +1605,6 @@ gst_collect_pads_default_compare_func (GstCollectPads * pads,
  * Default GstCollectPads event handling that elements should always
  * chain up to to ensure proper operation.  Element might however indicate
  * event should not be forwarded downstream.
- *
- * Since: 0.11.x
  */
 gboolean
 gst_collect_pads_event_default (GstCollectPads * pads, GstCollectData * data,
@@ -1792,7 +1698,6 @@ gst_collect_pads_event_default (GstCollectPads * pads, GstCollectData * data,
     case GST_EVENT_SEGMENT:
     {
       GstSegment seg;
-      gint cmp_res;
 
       GST_COLLECT_PADS_STREAM_LOCK (pads);
 
@@ -1807,29 +1712,21 @@ gst_collect_pads_event_default (GstCollectPads * pads, GstCollectData * data,
         goto newsegment_done;
       }
 
+      /* need to update segment first */
       data->segment = seg;
       GST_COLLECT_PADS_STATE_SET (data, GST_COLLECT_PADS_STATE_NEW_SEGMENT);
 
+      /* now we can use for e.g. running time */
+      seg.position =
+          gst_collect_pads_clip_time (pads, data, seg.start + seg.offset);
+      /* update again */
+      data->segment = seg;
+
       /* default muxing functionality */
       if (!buffer_func)
         goto newsegment_done;
 
-      /* If oldest time is not known, or current pad got newsegment;
-       * recalculate the state */
-      if (!pads->priv->earliest_data || pads->priv->earliest_data == data) {
-        gst_collect_pads_recalculate_full (pads);
-        goto newsegment_done;
-      }
-
-      /* Check if the waiting state of the pad should change. */
-      cmp_res =
-          pads->priv->compare_func (pads, data, seg.start,
-          pads->priv->earliest_data, pads->priv->earliest_time,
-          pads->priv->compare_user_data);
-
-      if (cmp_res > 0)
-        /* Stop waiting */
-        gst_collect_pads_set_waiting (pads, data, FALSE);
+      gst_collect_pads_handle_position_update (pads, data, seg.position);
 
     newsegment_done:
       GST_COLLECT_PADS_STREAM_UNLOCK (pads);
@@ -1837,9 +1734,31 @@ gst_collect_pads_event_default (GstCollectPads * pads, GstCollectData * data,
        * accumulated and this is certainly not what we want. */
       goto eat;
     }
-    case GST_EVENT_CAPS:
+    case GST_EVENT_GAP:
+    {
+      GstClockTime start, duration;
+
+      GST_COLLECT_PADS_STREAM_LOCK (pads);
+
+      gst_event_parse_gap (event, &start, &duration);
+      /* FIXME, handle reverse playback case */
+      if (GST_CLOCK_TIME_IS_VALID (duration))
+        start += duration;
+      /* we do not expect another buffer until after gap,
+       * so that is our position now */
+      data->segment.position = gst_collect_pads_clip_time (pads, data, start);
+
+      gst_collect_pads_handle_position_update (pads, data,
+          data->segment.position);
+
+      GST_COLLECT_PADS_STREAM_UNLOCK (pads);
+      goto eat;
+    }
     case GST_EVENT_STREAM_START:
-    case GST_EVENT_STREAM_CONFIG:
+      /* drop stream start events, element must create its own start event,
+       * we can't just forward the first random stream start event we get */
+      goto eat;
+    case GST_EVENT_CAPS:
       goto eat;
     default:
       /* forward other events */
@@ -1920,7 +1839,7 @@ pad_removed:
 
 /**
  * gst_collect_pads_query_default:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @data: collect data of corresponding pad
  * @query: query being processed
  * @discard: process but do not send event downstream
@@ -1928,8 +1847,6 @@ pad_removed:
  * Default GstCollectPads query handling that elements should always
  * chain up to to ensure proper operation.  Element might however indicate
  * query should not be forwarded downstream.
- *
- * Since: 0.11.x
  */
 gboolean
 gst_collect_pads_query_default (GstCollectPads * pads, GstCollectData * data,
@@ -2092,7 +2009,11 @@ gst_collect_pads_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 
   /* update segment last position if in TIME */
   if (G_LIKELY (data->segment.format == GST_FORMAT_TIME)) {
-    GstClockTime timestamp = GST_BUFFER_TIMESTAMP (buffer);
+    GstClockTime timestamp;
+
+    timestamp = GST_BUFFER_DTS (buffer);
+    if (!GST_CLOCK_TIME_IS_VALID (timestamp))
+      timestamp = GST_BUFFER_PTS (buffer);
 
     if (GST_CLOCK_TIME_IS_VALID (timestamp))
       data->segment.position = timestamp;
@@ -2154,7 +2075,9 @@ gst_collect_pads_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 
 unlock_done:
   GST_COLLECT_PADS_STREAM_UNLOCK (pads);
-  unref_data (data);
+  /* data is definitely NULL if pad_removed goto was run. */
+  if (data)
+    unref_data (data);
   if (buffer)
     gst_buffer_unref (buffer);
   return ret;
